/*
    gamglmb.cpp
    Bayesian Gamma generalized linear model.
    $Id: gamglm.cpp,v 1.4 2014/10/27 12:01:07 daichi Exp $

*/
#include <cstdio>
#include <cstdlib>
#include <unistd.h>
#include <vector>
#include "read.h"
#include "learn.h"
#include "id.h"
#define	 EPS_DEFAULT	0.2
#define	 SIGMA_DEFAULT	0.1
#define	 ITERS_DEFAULT	1

static char opts[] = "s:I:h";

void
usage ()
{
	fprintf(stderr, "gamglm: Bayesian Gamma generalized linear model.\n");
	fprintf(stderr, "$Id: gamglm.cpp,v 1.4 2014/10/27 12:01:07 daichi Exp $\n");
	fprintf(stderr, "usage: gamglm [-I iter] [-e eps] [-s sigma] TRAIN MODEL\n");
	exit(0);
}

int
main (int argc, char *argv[])
{
	vector<double> y;
	vector<vector<int> > features;
	vector<double> wa, wb;
	char *train, *model;
	double eps = EPS_DEFAULT;
	double sigma = SIGMA_DEFAULT;
	int c, iters = ITERS_DEFAULT;

	while ((c = getopt(argc, argv, opts)) != -1)
	{
		switch (c) {
		case 'I' : iters = atoi(optarg); break;
		case 's' : sigma = atof(optarg); break;
		case 'e' : eps = atof(optarg); break;
		case 'h' : usage (); break;
		default  : break;
		}
	}
	if (argc - optind != 2)
		usage ();
	else {
		train = argv[optind];
		model = argv[optind + 1];
	}

	read_data (train, y, features);
	learn_model (y, features, wa, wb, iters, eps, sigma);
	save_model (model, wa, wb);

	return 0;
}

