/*
    id.cpp
    feature dictionary functions.
    $Id: id.cpp,v 1.2 2014/10/27 01:48:19 daichi Exp $
    
*/
#include <iostream>
#include <fstream>
#include <string>
#include "id.h"
#include "util.h"

using namespace std;

static int id = 0;
static hash_map<string,int> hash;
static hash_map<int,string> rhash;

int
ids ()
{
	return id;
}

int
stoid (string &s)
{
	dictionary::iterator it;

	if ((it = hash.find(s)) == hash.end())
	{
		hash.insert (cons(s, id));
		rhash.insert (cons(id, s));
		return id++;
	} else {
		return it->second;
	}
}

int
fstoid (string &s)	/* fixed dictionary */
{
	dictionary::iterator it;

	if ((it = hash.find(s)) == hash.end())
	{
		return -1;
	} else {
		return it->second;
	}
}

void
load_dict (char *file)
{
	int n;
	string s;
	ifstream ifs (strconcat(file, ".dic"));

	if (ifs.fail())
	{
		error("cannot open %s for dictionary.", file);
		return;
	}

	for (n = 0; getline(ifs, s, '\n'); n++)
	{
		if (n > 0)	/* ignore bias term, id >= 0 */
		{
			hash.insert (cons(s, id));
			rhash.insert (cons(id, s));
			id++;
		}
	}

	ifs.close();
}

void
save_dict (char *file)
{
	dictionary::iterator it;
	FILE *fp;
	int f;
	
	if ((fp = fopen(strconcat(file, ".dic"), "w")) == NULL)
	{
		error("cannot open %s for dictionary.", file);
		return;
	}

	fprintf(fp, "bias\n");
	for (f = 0; f < id; f++)
	{
		fprintf(fp, "%s\n", rhash[f].c_str());
	}
	
	fclose(fp);
}

	

#if 0

int
main (int argc, char *argv[])
{
	int i;
	string tok;
	
	for (i = 1; i < argc; i++) {
		tok = argv[i];
		printf("id[%s] = %d\n", tok.c_str(), stoid(tok));
	}

	printf("lexicon = %d\n", id);
}


#endif
