/*
    random.cpp
    $Id: random.cpp,v 1.1 2014/12/19 02:40:03 daichi Exp $

 */
#include <cmath>
#include "random.h"
#include "gslrand.h"

double
sgamrand (double a)
{
	double x, y, z;
	double u, v, w, b, c, e;
	int accept = 0;
	if (a <= 1)
	{
		/* a <= 1. Johnk's generator. Devroye (1986) p.418 */
		e = exprand();
		do {
			x = pow(RANDOM, 1 / a);
			y = pow(RANDOM, 1 / (1 - a));
		} while (x + y > 1);
		return (e * x / (x + y));
	} else {
		/* a > 1. Best's rejection algorithm. Devroye (1986) p.410 */
		b = a - 1;
		c = 3 * a - 0.75;
		while (accept != 1)
		{
			/* generate */
			u = RANDOM;
			v = RANDOM;
			w = u * (1 - u);
			y = sqrt(c / w) * (u - 0.5);
			x = b + y;
			if (x >= 0)
			{
				z = 64 * w * w * w * v * v;
				if (z <= 1 - (2 * y * y) / x)
				{
					accept = 1;
				} else {
					if (log(z) <= 2 * (b * log(x / b) - y))
						accept = 1;
				}
			}
		}
		return x;
	}
}

double
gamrand (double a, double b)
{
	return sgamrand(a) / b;
}

double
exprand (void)
{
	return (- log(RANDOM));
}

double
betarand (double a, double b)
{
	double x = sgamrand(a), y = sgamrand(b);
	return x / (x + y);
}

int
irand (int n)
{
	return int(RANDOM * n);
}

int
bernoulli (double p)
{
	if (RANDOM < p)
		return 1;
	else
		return 0;
}

double
gampdf (double x, double a, double b)
{
	/* b^a / Gamma(a) * x^(a-1) * exp(-bx) */
	/* NOTE: b is 1/b in gampdf of MATLAB. */
	return exp(lgampdf(x, a, b));
}

double
lgampdf (double x, double a, double b)
{
	return a * log(b) - lgamma(a) + (a - 1) * log(x) - b * x;
}

double
lexppdf (double x, double b)
{
	return log(b) - b * x;
}

void
shuffle (int *v, int n)
{
	int i, j, t;

	for (i = n - 1; i >= 0; i--)
	{
		j = (int)(RANDOM * (i + 1));
		t = v[i];
		v[i] = v[j];
		v[j] = t;
	}
}

#if 0
#include <cstdio>
#include <cstdlib>

using namespace std;

int
main (int argc, char *argv[])
{
	int i, n = atoi(argv[1]);
	double a = atof(argv[2]), b = atof(argv[3]);

	for (i = 0; i < n; i++)
		printf("gampdf(%g|%g,%g) = %g\n",
		       i + 0.0, a, b, gampdf(i+0.0, a, b));
}

#endif
