/*
    util.cpp
    utility functions in C++.
    $Id: util.cpp,v 1.1 2014/10/23 08:18:33 daichi Exp $
*/
#include <deque>
#include <vector>
#include <string>
#include <numeric>
#include <sstream>
#include <cstdio>
#include <cstdlib>
#include <cstdarg>
#include <cstring>
#include <cmath>
#include <boost/tokenizer.hpp>
#include "util.h"

using namespace std;

typedef boost::tokenizer<boost::char_separator<char> > Tokenizer;
typedef Tokenizer::iterator TokIter;
boost::char_separator<char> sep(" \t");

vector<string>
parse_line (string &line)
{
	Tokenizer tok (line, sep);
	vector<string> ss;

	for (TokIter p = tok.begin(); p != tok.end(); p++)
	{
		if (*p == "//")			/* ɤФ */
			break;	
		if ((*p).substr(0,2) == "->")	/* ->  */
			continue;
		ss.push_back (*p);
	}

	return ss;
}

template <class T>
T
unshift (deque<T> &v)
{
	T x = v[0];
	v.pop_front ();
	return x;
}

template <class T>
void
push_front (vector<T> &v, T &x)
{
	v.insert (v.begin(), x);
}

template <class T>
void
pop_front (vector<T> &v)
{
	v.erase (v.begin());
}

double
sum (vector<double> &v)
{
	return accumulate(v.begin(), v.end(), 0.0);
}

int
vmax (double *v, int n)
{
	int i, m = 0;
	for (i = 0; i < n; i++)
		if (v[i] > v[m])
			m = i;
	return m;
}

int
vmax (vector<int> &v)
{
	vector<int>::iterator it;
	int val;
	
	for (it = v.begin(); it != v.end(); it++)
	{
		if (it == v.begin())
			val = *it;
		else {
			if (*it > val)
				val = *it;
		}
	}
	return val;
}
	
void
vec_normalize (double *v, int n)
{
	double z = 0;
	int i;

	for (i = 0; i < n; i++)
		z += v[i];
	for (i = 0; i < n; i++)
		if (v[i] > 0)
			v[i] /= z;
}

void
matrix_normalize (double **matrix, int rows, int cols, int columnwise)
{
	double z;
	int i, j;

	if (columnwise)
	{
		for (j = 0; j < cols; j++)
		{
			z = 0;
			for (i = 0; i < rows; i++)
				z += matrix[i][j];
			for (i = 0; i < rows; i++)
				matrix[i][j] /= z;
		}
	} else {
		for (i = 0; i < rows; i++)
		{
			z = 0;
			for (j = 0; j < cols; j++)
				z += matrix[i][j];
			for (j = 0; j < cols; j++)
				matrix[i][j] /= z;
		}
	}
}	

double
logsumexp (double x, double y, int init)
{
	double vmin = min(x, y);
	double vmax = max(x, y);
	
	if (init) return y;
	if (x == y) return x + log(2.0);
	if (vmax > vmin + 50)
		return vmax;
	else
		return vmax + log(1 + exp(vmin - vmax));
}

bool
isspaces (const string &line)
{
	string::const_iterator it;

	for (it = line.begin(); it != line.end(); it++)
		if (!isspace(*it))
			return false;
	return true;
}

char *
strconcat (const char *s, const char *t)
{
	static char z[BUFSIZ];
	strcpy(z, s);
	strcat(z, t);
	return strdup(z);
}

string
join (vector<string> &words, string &delim)
{
	vector<string>::iterator it = words.begin();
	string s = *it;

	for (it++; it != words.end(); it++)
	{
		s += delim + *it;
	}

	return s;
}

bool
converged (double u, double v, double threshold)
{
	/* return true if |u - v|/|u| < threshold */
	if (fabs((u - v) / v) < threshold)
		return true;
	else
		return false;
}

double *
drealloc (double *v, int size, int old)
{
	int i;

	if ((v = (double *)realloc(v, size * sizeof(double)))
	    == NULL)
	{
		perror("realloc");
		exit(1);
	}
	for (i = old; i < size; i++)
		v[i] = 0.0;
	
	return v;
}
	
int
eprintf (const char *fmt, ...)
{
	int n;
	va_list ap;

	va_start(ap, fmt);
	n = vfprintf(stderr, fmt, ap);
	va_end(ap);
	fflush(stderr);

	return n;
}
	
int
error (const char *fmt, ...)
{
	int n;
	va_list ap;
	
	va_start(ap, fmt);
	n = vfprintf(stderr, fmt, ap);
	va_end(ap);
	fprintf(stderr, "\n");

	return n;
}

bool
fexists (char *file)
{
	FILE *fp;

	if ((fp = fopen(file, "r")) == NULL)
		return false;
	else {
		fclose(fp);
		return true;
	}
}

double
stof (string &s)
{
	stringstream ss(s);
	double v;
	
	ss >> v;

	return v;
}

template pair<int,int> unshift (deque<pair<int,int> > &v);

double
hashsum (hash_map<int,double> &h)
{
	hash_map<int,double>::iterator it;
	double z = 0;

	for (it = h.begin(); it != h.end(); it++)
		z += it->second;

	return z;
}

double
hashsum (hash_map<pair<int,int>,double> &h)
{
	hash_map<pair<int,int>,double>::iterator it;
	double z = 0;

	for (it = h.begin(); it != h.end(); it++)
		z += it->second;

	return z;
}


#if 0
#include <cstdio>

int
main (int argc, char *argv[])
{
	vector<int> v;
	int i;

	for (i = 1; i < argc; i++) {
		v.push_back (atoi(argv[i]));
		printf("%d\n", atoi(argv[i]));
	}
	
	printf("max = %d\n", vmax(v));
}

#endif
